#include <stdio.h>
#include <stdlib.h>
#include "logger.h"
#include "snes9x.h"
#include "soundux.h"
#include "ppu.h"
#include "movie.h"

int dumpstreams = 0;
int maxframes = -1;
int ignoreframes = 0;

static int resetno = 0;
static int framecounter = 0;

static int drift = 0;

FILE *video=NULL, *audio=NULL;
char autodemo[128] = "";

void ResetLogger()
{
	char buffer[256*224*4];

	if (!dumpstreams)
		return;

	framecounter = 0;
	drift=0;

	if (video)
		fclose(video);
	if (audio)
		fclose(audio);

	sprintf(buffer, "videostream%d.dat", resetno);
	video = fopen(buffer, "wb");
	if (!video)
	{
		printf("Opening %s failed. Logging cancelled.\n", buffer);
		return;
	}
	
	sprintf(buffer, "audiostream%d.dat", resetno);
	audio = fopen(buffer, "wb");
	if (!audio)
	{
		printf("Opening %s failed. Logging cancelled.\n", buffer);
		fclose(video);
		return;
	}

	char *logo = getenv("LOGO");
	if (!logo)
		logo = "logo.dat";
	FILE *l = fopen(logo, "rb");
	if (l)
	{
		const int soundsize = (so.sixteen_bit ? 2 : 1)*(so.stereo?2:1)*so.playback_rate * Settings.FrameTime / 1000000;
		printf("Soundsize: %d\n", soundsize);
		while (!feof(l))
		{
			if (fread(buffer, 1024,224, l) != 224)
				break;
			VideoLogger(buffer, 256, 224, 24);
			memset(buffer, 0, soundsize);
			AudioLogger(buffer, soundsize);
		}
		fclose(l);
	}
	resetno++;
}


void VideoLogger(void *pixels, int width, int height, int depth)
{
	int fc = S9xMovieGetFrameCounter();
	if (fc > 0)
		framecounter = fc;
	else
		framecounter++;	

	if (video)
	{
		fwrite(pixels, 256*4, 224, video);
		fflush(video);
		fflush(audio);
		drift++;

		if (maxframes > 0 && __builtin_expect(framecounter >= maxframes, 0))
		{
			printf("-maxframes hit\ndrift:%d\n",drift);
			S9xExit();
		}	
		
	}

	if (Settings.DisplayPressedKeys)
	{
                fprintf(stderr, "%d %s %s %s %s %s %s %c%c%c%c%c%c        \r", framecounter,
		IPPU.Joypads[0] & SNES_START_MASK ? "Start" : "_____",
		IPPU.Joypads[0] & SNES_SELECT_MASK ? "Select" : "______",
                IPPU.Joypads[0] & SNES_UP_MASK ? "Up" : "__",
		IPPU.Joypads[0] & SNES_DOWN_MASK ? "Down" : "____",
		IPPU.Joypads[0] & SNES_LEFT_MASK ? "Left" : "____",
		IPPU.Joypads[0] & SNES_RIGHT_MASK ? "Right" : "_____",
		IPPU.Joypads[0] & SNES_A_MASK ? 'A':'_',
		IPPU.Joypads[0] & SNES_B_MASK ? 'B':'_',
                IPPU.Joypads[0] & SNES_X_MASK ? 'X':'_',
                IPPU.Joypads[0] & SNES_Y_MASK ? 'Y':'_',  
		IPPU.Joypads[0] & SNES_TL_MASK ? 'L':'_',
		IPPU.Joypads[0] & SNES_TR_MASK ? 'R':'_');

	}
}


void AudioLogger(void *samples, int length)
{
	if (audio)
		fwrite(samples, 1, length, audio);
	drift--;
}
